@@ -65,6 +65,10 @@ typedef struct {
 	gboolean busy;
 	gboolean init_run;
 	guint validate_id;
+
+	char *last_validation_error;
+
+	GHashTable *inter_page_hash;
 } NMConnectionEditor;
 
 typedef struct {
@@ -74,6 +78,11 @@ typedef struct {
 	void (*done)  (NMConnectionEditor *editor, gint result, GError *error);
 } NMConnectionEditorClass;
 
+typedef enum {
+	/* Add item for inter-page changes here */
+	INTER_PAGE_CHANGE_WIFI_MODE = 1,
+} InterPageChangeType;
+
 GType               nm_connection_editor_get_type (void);
 NMConnectionEditor *nm_connection_editor_new (GtkWindow *parent_window,
                                               NMConnection *connection,
@@ -98,4 +107,12 @@ void                nm_connection_editor_warning (GtkWindow *parent,
                                                   const char *format,
                                                   ...);
 
+void               nm_connection_editor_inter_page_set_value (NMConnectionEditor *editor,
+                                                              InterPageChangeType type,
+                                                              gpointer value);
+gboolean           nm_connection_editor_inter_page_get_value (NMConnectionEditor *editor,
+                                                              InterPageChangeType type,
+                                                              gpointer *value);
+void               nm_connection_editor_inter_page_clear_data (NMConnectionEditor *editor);
+
 #endif
