@@ -430,17 +430,19 @@ finish_setup (CEPageBond *self, gpointer unused, GError *error, gpointer user_da
 }
 
 CEPage *
-ce_page_bond_new (NMConnection *connection,
-				  GtkWindow *parent_window,
-				  NMClient *client,
+ce_page_bond_new (NMConnectionEditor *editor,
+                  NMConnection *connection,
+                  GtkWindow *parent_window,
+                  NMClient *client,
                   NMRemoteSettings *settings,
-				  const char **out_secrets_setting_name,
-				  GError **error)
+                  const char **out_secrets_setting_name,
+                  GError **error)
 {
 	CEPageBond *self;
 	CEPageBondPrivate *priv;
 
 	self = CE_PAGE_BOND (ce_page_new (CE_TYPE_PAGE_BOND,
+	                                  editor,
 	                                  connection,
 	                                  parent_window,
 	                                  client,
@@ -560,17 +562,16 @@ ui_to_setting (CEPageBond *self)
 }
 
 static gboolean
-validate (CEPage *page, NMConnection *connection, GError **error)
+ce_page_validate_v (CEPage *page, NMConnection *connection, GError **error)
 {
 	CEPageBond *self = CE_PAGE_BOND (page);
 	CEPageBondPrivate *priv = CE_PAGE_BOND_GET_PRIVATE (self);
-	const char *primary;
 
-	if (!CE_PAGE_CLASS (ce_page_bond_parent_class)->validate (page, connection, error))
+	if (!CE_PAGE_CLASS (ce_page_bond_parent_class)->ce_page_validate_v (page, connection, error))
 		return FALSE;
 
-	primary = gtk_entry_get_text (priv->primary);
-	if (primary && *primary && !nm_utils_iface_valid_name (primary))
+	if (!ce_page_interface_name_valid (gtk_entry_get_text (priv->primary),
+	                                   _("primary"), error))
 		return FALSE;
 
 	ui_to_setting (self);
@@ -597,7 +598,7 @@ ce_page_bond_class_init (CEPageBondClass *bond_class)
 	g_type_class_add_private (object_class, sizeof (CEPageBondPrivate));
 
 	/* virtual methods */
-	parent_class->validate = validate;
+	parent_class->ce_page_validate_v = ce_page_validate_v;
 
 	master_class->connection_added = connection_added;
 	master_class->connection_removed = connection_removed;
