@@ -92,7 +92,8 @@ finish_setup (CEPageBridgePort *self, gpointer unused, GError *error, gpointer u
 }
 
 CEPage *
-ce_page_bridge_port_new (NMConnection *connection,
+ce_page_bridge_port_new (NMConnectionEditor *editor,
+                         NMConnection *connection,
                          GtkWindow *parent_window,
                          NMClient *client,
                          NMRemoteSettings *settings,
@@ -103,6 +104,7 @@ ce_page_bridge_port_new (NMConnection *connection,
 	CEPageBridgePortPrivate *priv;
 
 	self = CE_PAGE_BRIDGE_PORT (ce_page_new (CE_TYPE_PAGE_BRIDGE_PORT,
+	                                         editor,
 	                                         connection,
 	                                         parent_window,
 	                                         client,
@@ -145,7 +147,7 @@ ui_to_setting (CEPageBridgePort *self)
 }
 
 static gboolean
-validate (CEPage *page, NMConnection *connection, GError **error)
+ce_page_validate_v (CEPage *page, NMConnection *connection, GError **error)
 {
 	CEPageBridgePort *self = CE_PAGE_BRIDGE_PORT (page);
 	CEPageBridgePortPrivate *priv = CE_PAGE_BRIDGE_PORT_GET_PRIVATE (self);
@@ -168,5 +170,5 @@ ce_page_bridge_port_class_init (CEPageBridgePortClass *bridge_port_class)
 	g_type_class_add_private (object_class, sizeof (CEPageBridgePortPrivate));
 
 	/* virtual methods */
-	parent_class->validate = validate;
+	parent_class->ce_page_validate_v = ce_page_validate_v;
 }
