@@ -583,7 +583,8 @@ finish_setup (CEPageDcb *self, gpointer unused, GError *error, gpointer user_dat
 }
 
 CEPage *
-ce_page_dcb_new (NMConnection *connection,
+ce_page_dcb_new (NMConnectionEditor *editor,
+                 NMConnection *connection,
                  GtkWindow *parent_window,
                  NMClient *client,
                  NMRemoteSettings *settings,
@@ -596,6 +597,7 @@ ce_page_dcb_new (NMConnection *connection,
 	NMSettingDcb *s_dcb;
 
 	self = CE_PAGE_DCB (ce_page_new (CE_TYPE_PAGE_DCB,
+	                                 editor,
 	                                 connection,
 	                                 parent_window,
 	                                 client,
@@ -727,7 +729,7 @@ ui_to_setting (CEPageDcb *self, NMSettingDcb *s_dcb)
 }
 
 static gboolean
-validate (CEPage *page, NMConnection *connection, GError **error)
+ce_page_validate_v (CEPage *page, NMConnection *connection, GError **error)
 {
 	CEPageDcb *self = CE_PAGE_DCB (page);
 	CEPageDcbPrivate *priv = CE_PAGE_DCB_GET_PRIVATE (self);
@@ -774,5 +776,5 @@ ce_page_dcb_class_init (CEPageDcbClass *security_class)
 	/* virtual methods */
 	object_class->dispose = dispose;
 
-	parent_class->validate = validate;
+	parent_class->ce_page_validate_v = ce_page_validate_v;
 }
