@@ -63,6 +63,8 @@ typedef struct {
 
 	GtkComboBox *method;
 	GtkListStore *method_store;
+	int normal_method_idx;
+	int hotspot_method_idx;
 
 	/* Addresses */
 	GtkWidget *addr_label;
@@ -151,7 +153,7 @@ ip6_private_init (CEPageIP6 *self, NMConnection *connection)
 	priv->method = GTK_COMBO_BOX (gtk_builder_get_object (builder, "ip6_method"));
 	cells = gtk_cell_layout_get_cells (GTK_CELL_LAYOUT (priv->method));
 	gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (priv->method), cells->data,
-								   "sensitive", METHOD_COL_ENABLED);
+	                               "sensitive", METHOD_COL_ENABLED);
 
 	priv->method_store = gtk_list_store_new (3, G_TYPE_STRING, G_TYPE_UINT, G_TYPE_BOOLEAN);
 
@@ -159,21 +161,21 @@ ip6_private_init (CEPageIP6 *self, NMConnection *connection)
 	gtk_list_store_set (priv->method_store, &iter,
 	                    METHOD_COL_NAME, _("Ignore"),
 	                    METHOD_COL_NUM, IP6_METHOD_IGNORE,
-						METHOD_COL_ENABLED, TRUE,
+	                    METHOD_COL_ENABLED, TRUE,
 	                    -1);
 
 	gtk_list_store_append (priv->method_store, &iter);
 	gtk_list_store_set (priv->method_store, &iter,
 	                    METHOD_COL_NAME, str_auto,
 	                    METHOD_COL_NUM, IP6_METHOD_AUTO,
-						METHOD_COL_ENABLED, TRUE,
+	                    METHOD_COL_ENABLED, TRUE,
 	                    -1);
 
 	gtk_list_store_append (priv->method_store, &iter);
 	gtk_list_store_set (priv->method_store, &iter,
 	                    METHOD_COL_NAME, str_auto_only,
 	                    METHOD_COL_NUM, IP6_METHOD_AUTO_ADDRESSES,
-						METHOD_COL_ENABLED, TRUE,
+	                    METHOD_COL_ENABLED, TRUE,
 	                    -1);
 
 	/* DHCP only used on Wi-Fi and ethernet for now */
@@ -183,7 +185,7 @@ ip6_private_init (CEPageIP6 *self, NMConnection *connection)
 		gtk_list_store_set (priv->method_store, &iter,
 		                    METHOD_COL_NAME, _("Automatic, DHCP only"),
 		                    METHOD_COL_NUM, IP6_METHOD_AUTO_DHCP_ONLY,
-							METHOD_COL_ENABLED, TRUE,
+		                    METHOD_COL_ENABLED, TRUE,
 		                    -1);
 	}
 
@@ -195,7 +197,7 @@ ip6_private_init (CEPageIP6 *self, NMConnection *connection)
 		gtk_list_store_set (priv->method_store, &iter,
 		                    METHOD_COL_NAME, _("Manual"),
 		                    METHOD_COL_NUM, IP6_METHOD_MANUAL,
-							METHOD_COL_ENABLED, TRUE,
+		                    METHOD_COL_ENABLED, TRUE,
 		                    -1);
 	}
 
@@ -208,14 +210,14 @@ ip6_private_init (CEPageIP6 *self, NMConnection *connection)
 		gtk_list_store_set (priv->method_store, &iter,
 		                    METHOD_COL_NAME, _("Link-Local Only"),
 		                    METHOD_COL_NUM, IP6_METHOD_LINK_LOCAL,
-							METHOD_COL_ENABLED, TRUE,
+		                    METHOD_COL_ENABLED, TRUE,
 		                    -1);
 
 		gtk_list_store_append (priv->method_store, &iter);
 		gtk_list_store_set (priv->method_store, &iter,
 		                    METHOD_COL_NAME, _("Shared to other computers"),
 		                    METHOD_COL_NUM, IP6_METHOD_SHARED,
-							METHOD_COL_ENABLED, FALSE,
+		                    METHOD_COL_ENABLED, FALSE,
 		                    -1);
 	}
 
@@ -983,7 +985,8 @@ finish_setup (CEPageIP6 *self, gpointer unused, GError *error, gpointer user_dat
 }
 
 CEPage *
-ce_page_ip6_new (NMConnection *connection,
+ce_page_ip6_new (NMConnectionEditor *editor,
+                 NMConnection *connection,
                  GtkWindow *parent_window,
                  NMClient *client,
                  NMRemoteSettings *settings,
@@ -995,6 +998,7 @@ ce_page_ip6_new (NMConnection *connection,
 	NMSettingConnection *s_con;
 
 	self = CE_PAGE_IP6 (ce_page_new (CE_TYPE_PAGE_IP6,
+	                                 editor,
 	                                 connection,
 	                                 parent_window,
 	                                 client,
@@ -1028,7 +1032,7 @@ ce_page_ip6_new (NMConnection *connection,
 }
 
 static gboolean
-ui_to_setting (CEPageIP6 *self)
+ui_to_setting (CEPageIP6 *self, GError **error)
 {
 	CEPageIP6Private *priv = CE_PAGE_IP6_GET_PRIVATE (self);
 	GtkTreeModel *model;
@@ -1092,8 +1096,7 @@ ui_to_setting (CEPageIP6 *self)
 		/* IP address */
 		gtk_tree_model_get (model, &tree_iter, COL_ADDRESS, &item, -1);
 		if (!item || !inet_pton (AF_INET6, item, &tmp_addr)) {
-			g_warning ("%s: IPv6 address '%s' missing or invalid!",
-			           __func__, item ? item : "<none>");
+			g_set_error (error, NMA_ERROR, NMA_ERROR_GENERIC, _("IPv6 address \"%s\" invalid"), item ? item : "");
 			g_free (item);
 			goto out;
 		}
@@ -1102,15 +1105,13 @@ ui_to_setting (CEPageIP6 *self)
 		/* Prefix */
 		gtk_tree_model_get (model, &tree_iter, COL_PREFIX, &item, -1);
 		if (!item) {
-			g_warning ("%s: IPv6 prefix '%s' missing!",
-			           __func__, item ? item : "<none>");
+			g_set_error_literal (error, NMA_ERROR, NMA_ERROR_GENERIC, _("IPv6 prefix \"%s\" missing"));
 			goto out;
 		}
 
 		prefix = strtoul (item, &end, 10);
 		if (!end || *end || prefix == 0 || prefix > 128) {
-			g_warning ("%s: IPv6 prefix '%s' invalid!",
-			           __func__, item ? item : "<none>");
+			g_set_error (error, NMA_ERROR, NMA_ERROR_GENERIC, _("IPv6 prefix \"%s\" invalid"), item);
 			g_free (item);
 			goto out;
 		}
@@ -1120,8 +1121,7 @@ ui_to_setting (CEPageIP6 *self)
 		gtk_tree_model_get (model, &tree_iter, COL_GATEWAY, &item, -1);
 		if (item && strlen (item)) {
 			if (!inet_pton (AF_INET6, item, &tmp_gw)) {
-				g_warning ("%s: IPv6 gateway '%s' missing or invalid!",
-				           __func__, item ? item : "<none>");
+				g_set_error (error, NMA_ERROR, NMA_ERROR_GENERIC, _("IPv6 gateway \"%s\" invalid"), item);
 				g_free (item);
 				goto out;
 			}
@@ -1156,6 +1156,7 @@ ui_to_setting (CEPageIP6 *self)
 			if (inet_pton (AF_INET6, stripped, &tmp_addr)) {
 				nm_setting_ip6_config_add_dns (priv->setting, &tmp_addr);
 			} else {
+				g_set_error (error, NMA_ERROR, NMA_ERROR_GENERIC, _("IPv6 DNS server \"%s\" invalid"), stripped);
 				g_strfreev (items);
 				goto out;
 			}
@@ -1208,22 +1209,110 @@ out:
 }
 
 static gboolean
-validate (CEPage *page, NMConnection *connection, GError **error)
+ce_page_validate_v (CEPage *page, NMConnection *connection, GError **error)
 {
 	CEPageIP6 *self = CE_PAGE_IP6 (page);
 	CEPageIP6Private *priv = CE_PAGE_IP6_GET_PRIVATE (self);
 
-	if (!ui_to_setting (self))
+	if (!ui_to_setting (self, error))
 		return FALSE;
 	return nm_setting_verify (NM_SETTING (priv->setting), NULL, error);
 }
 
+static gboolean
+get_iter_for_method (GtkTreeModel *model, int column, GtkTreeIter *iter)
+{
+	int col;
+
+	if (gtk_tree_model_get_iter_first (model, iter)) {
+		do {
+			gtk_tree_model_get (model, iter, METHOD_COL_NUM, &col, -1);
+			if (col == column)
+				return TRUE;
+		} while (gtk_tree_model_iter_next (model, iter));
+	}
+	return FALSE;
+}
+
+static void
+toggle_method_sensitivity (CEPage *page, int column, gboolean sensitive)
+{
+	CEPageIP6 *self = CE_PAGE_IP6 (page);
+	CEPageIP6Private *priv = CE_PAGE_IP6_GET_PRIVATE (self);
+	GtkTreeModel *model = GTK_TREE_MODEL (priv->method_store);
+	GtkTreeIter iter;
+
+	if (get_iter_for_method (model, column, &iter))
+		gtk_list_store_set (priv->method_store, &iter, METHOD_COL_ENABLED, sensitive, -1);
+}
+
+static gboolean
+get_method_sensitivity (CEPage *page, int column)
+{
+	CEPageIP6 *self = CE_PAGE_IP6 (page);
+	CEPageIP6Private *priv = CE_PAGE_IP6_GET_PRIVATE (self);
+	GtkTreeModel *model = GTK_TREE_MODEL (priv->method_store);
+	GtkTreeIter iter;
+	gboolean sensitive = FALSE;
+
+	if (get_iter_for_method (model, column, &iter))
+		gtk_tree_model_get (GTK_TREE_MODEL (priv->method_store), &iter, METHOD_COL_ENABLED, &sensitive, -1);
+	return sensitive;
+}
+
+static void
+change_method_combo (CEPage *page, gboolean is_hotspot)
+{
+	CEPageIP6 *self = CE_PAGE_IP6 (page);
+	CEPageIP6Private *priv = CE_PAGE_IP6_GET_PRIVATE (self);
+
+	/* Store previous active method */
+	if (get_method_sensitivity (page, IP6_METHOD_AUTO))
+		priv->normal_method_idx = gtk_combo_box_get_active (priv->method);
+	else
+		priv->hotspot_method_idx = gtk_combo_box_get_active (priv->method);
+
+	/* Set active method */
+	if (is_hotspot) {
+		if (priv->hotspot_method_idx != -1)
+			gtk_combo_box_set_active (priv->method, priv->hotspot_method_idx);
+		else
+			gtk_combo_box_set_active (priv->method, IP6_METHOD_IGNORE);
+	} else {
+		if (priv->normal_method_idx != -1)
+			gtk_combo_box_set_active (priv->method, priv->normal_method_idx);
+	}
+
+	toggle_method_sensitivity (page, IP6_METHOD_AUTO, !is_hotspot);
+	toggle_method_sensitivity (page, IP6_METHOD_AUTO_ADDRESSES, !is_hotspot);
+	toggle_method_sensitivity (page, IP6_METHOD_AUTO_DHCP_ONLY, !is_hotspot);
+	toggle_method_sensitivity (page, IP6_METHOD_MANUAL, !is_hotspot);
+	toggle_method_sensitivity (page, IP6_METHOD_LINK_LOCAL, !is_hotspot);
+}
+
+static gboolean
+inter_page_change (CEPage *page)
+{
+	gpointer wifi_mode_ap;
+
+	if (nm_connection_editor_inter_page_get_value (page->editor, INTER_PAGE_CHANGE_WIFI_MODE, &wifi_mode_ap)) {
+		/* For Wi-Fi AP mode restrict IPv6 methods to ignore */
+		if (GPOINTER_TO_UINT (wifi_mode_ap))
+			change_method_combo (page, TRUE);
+		else
+			change_method_combo (page, FALSE);
+	}
+	return TRUE;
+}
+
 static void
 ce_page_ip6_init (CEPageIP6 *self)
 {
 	CEPageIP6Private *priv = CE_PAGE_IP6_GET_PRIVATE (self);
 
 	priv->last_column = -1;
+	priv->normal_method_idx = -1;
+	priv->hotspot_method_idx = -1;
 }
 
 static void
@@ -1253,6 +1342,7 @@ ce_page_ip6_class_init (CEPageIP6Class *ip6_class)
 	g_type_class_add_private (object_class, sizeof (CEPageIP6Private));
 
 	/* virtual methods */
-	parent_class->validate = validate;
+	parent_class->ce_page_validate_v = ce_page_validate_v;
+	parent_class->inter_page_change = inter_page_change;
 	object_class->dispose = dispose;
 }
