@@ -265,7 +265,8 @@ finish_setup (CEPagePpp *self, gpointer unused, GError *error, gpointer user_dat
 }
 
 CEPage *
-ce_page_ppp_new (NMConnection *connection,
+ce_page_ppp_new (NMConnectionEditor *editor,
+                 NMConnection *connection,
                  GtkWindow *parent_window,
                  NMClient *client,
                  NMRemoteSettings *settings,
@@ -277,6 +278,7 @@ ce_page_ppp_new (NMConnection *connection,
 	NMSettingConnection *s_con;
 
 	self = CE_PAGE_PPP (ce_page_new (CE_TYPE_PAGE_PPP,
+	                                 editor,
 	                                 connection,
 	                                 parent_window,
 	                                 client,
@@ -360,7 +362,7 @@ ui_to_setting (CEPagePpp *self)
 }
 
 static gboolean
-validate (CEPage *page, NMConnection *connection, GError **error)
+ce_page_validate_v (CEPage *page, NMConnection *connection, GError **error)
 {
 	CEPagePpp *self = CE_PAGE_PPP (page);
 	CEPagePppPrivate *priv = CE_PAGE_PPP_GET_PRIVATE (self);
@@ -394,6 +396,6 @@ ce_page_ppp_class_init (CEPagePppClass *ppp_class)
 	g_type_class_add_private (object_class, sizeof (CEPagePppPrivate));
 
 	/* virtual methods */
-	parent_class->validate = validate;
+	parent_class->ce_page_validate_v = ce_page_validate_v;
 	object_class->dispose = dispose;
 }
