@@ -44,9 +44,9 @@ destroy (WirelessSecurity *parent)
 }
 
 static gboolean
-validate (WirelessSecurity *parent, const GByteArray *ssid)
+validate (WirelessSecurity *parent, GError **error)
 {
-	return ws_802_1x_validate (parent, "dynamic_wep_auth_combo");
+	return ws_802_1x_validate (parent, "dynamic_wep_auth_combo", error);
 }
 
 static void
@@ -116,6 +116,7 @@ ws_dynamic_wep_new (NMConnection *connection,
 		return NULL;
 
 	parent->adhoc_compatible = FALSE;
+	parent->hotspot_compatible = FALSE;
 
 	widget = ws_802_1x_auth_combo_init (parent,
 	                                    "dynamic_wep_auth_combo",
